@@ -1,8 +1,8 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2017 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,14 +23,7 @@
  +-------------------------------------------------------------------------+
 */
 
-/* do NOT run this script through a web browser */
-if (!isset($_SERVER['argv'][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
-	die('<br><strong>This script is only meant to run at the command line.</strong>');
-}
-
-$no_http_headers = true;
-
-include(dirname(__FILE__) . '/../include/global.php');
+require(__DIR__ . '/../include/cli_check.php');
 
 /* process calling arguments */
 $parms = $_SERVER['argv'];
@@ -39,13 +32,14 @@ array_shift($parms);
 global $debug;
 
 $debug = false;
+$local = false;
 $form  = '';
 $start = time();
 
-if (sizeof($parms)) {
+if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
-			list($arg, $value) = explode('=', $parameter);
+			list($arg, $value) = explode('=', $parameter, 2);
 		} else {
 			$arg = $parameter;
 			$value = '';
@@ -56,32 +50,50 @@ if (sizeof($parms)) {
 			case '--debug':
 				$debug = true;
 				break;
+			case '--local':
+				$local = true;
+				break;
 			case '--version':
 			case '-V':
 			case '-v':
 				display_version();
+				exit(0);
 			case '--help':
 			case '-H':
 			case '-h':
 				display_help();
-				exit;
+				exit(0);
 			default:
 				print "ERROR: Invalid Parameter " . $parameter . "\n\n";
 				display_help();
-				exit;
+				exit(1);
 		}
 	}
 }
 
-echo "Analyzing All Cacti Database Tables\n";
+print "NOTE: Analyzing All Cacti Database Tables\n";
+
+if (!$local && $config['poller_id'] > 1) {
+	db_switch_remote_to_main();
+
+	print "NOTE: Repairing Tables for Main Database" . PHP_EOL;
+} else {
+	print "NOTE: Repairing Tables for Local Database" . PHP_EOL;
+}
 
 $tables = db_fetch_assoc('SHOW TABLES FROM `' . $database_default . '`');
 
-if (sizeof($tables)) {
+if (cacti_sizeof($tables)) {
 	foreach($tables AS $table) {
-		echo "Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "'";
-		$status = db_execute('ANALYZE TABLE ' . $table['Tables_in_' . $database_default] . $form);
-		echo ($status == 0 ? ' Failed' : ' Successful') . "\n";
+		if (db_binlog_enabled()) {
+			print "NOTE: Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "' without writing to the binlog";
+			$status = db_execute('ANALYZE TABLE NO_WRITE_TO_BINLOG ' . $table['Tables_in_' . $database_default] . $form);
+		} else {
+			print "NOTE: Analyzing Table -> '" . $table['Tables_in_' . $database_default] . "'";
+			$status = db_execute('ANALYZE TABLE ' . $table['Tables_in_' . $database_default] . $form);
+		}
+
+		print ($status == 0 ? ' Failed' : ' Successful') . "\n";
 	}
 
 	cacti_log('ANALYSIS STATS: Analyzing Cacti Tables Complete.  Total time ' . (time() - $start) . ' seconds.', false, 'SYSTEM');
@@ -89,17 +101,18 @@ if (sizeof($tables)) {
 
 /*  display_version - displays version information */
 function display_version() {
-	$version = get_cacti_version();
-	echo "Cacti Analyze Database Utility, Version $version, " . COPYRIGHT_YEARS . "\n";
+	$version = get_cacti_cli_version();
+	print "Cacti Analyze Database Utility, Version $version, " . COPYRIGHT_YEARS . "\n";
 }
 
 /*	display_help - displays the usage of the function */
 function display_help () {
 	display_version();
 
-	echo "\nusage: analyze_database.php [-d|--debug]\n\n";
-	echo "A utility to recalculate the cardinality of indexes within the Cacti database.\n";
-	echo "It's important to periodically run this utility expecially on larger systems.\n\n";
-	echo "Optional:\n";
-	echo "-d | --debug - Display verbose output during execution\n\n";
+	print "\nusage: analyze_database.php [-d|--debug]\n\n";
+	print "A utility to recalculate the cardinality of indexes within the Cacti database.\n";
+	print "It's important to periodically run this utility especially on larger systems.\n\n";
+	print "Optional:\n";
+	print "     --local   - Perform the action on the Remote Data Collector if run from there\n";
+	print "-d | --debug   - Display verbose output during execution\n\n";
 }
