@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -30,6 +30,12 @@ if ($config['poller_id'] > 1) {
 	exit(1);
 }
 
+$storage_location = read_config_option('storage_location');
+if ($storage_location > 0) {
+	print 'FATAL: This utility is designed for local RRDfile storage and is not compatible with the RRDProxy.' . PHP_EOL;
+	exit(1);
+}
+
 define('PHP_DEOL', PHP_EOL . PHP_EOL);
 
 $host_id          = false;
@@ -43,7 +49,7 @@ array_shift($parms);
 if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
-			list($arg, $value) = explode('=', $parameter);
+			list($arg, $value) = explode('=', $parameter, 2);
 		} else {
 			$arg = $parameter;
 			$value = '';
@@ -197,7 +203,7 @@ foreach ($data_sources as $info) {
 			struct_debug("NOTE: New Directory '$new_base_path' Created for RRD Files");
 
 			if ($config['cacti_server_os'] != 'win32') {
-				if (recursive_chown($new_base_path, $owner_id) && recursive_chgrp($new_base_path, $group_id)) {
+				if (sp_recursive_chown($new_base_path, $owner_id) && sp_recursive_chgrp($new_base_path, $group_id)) {
 					struct_debug("NOTE: New Directory '$new_base_path' Permissions Set");
 				} else {
 					print "FATAL: Could not Set Permissions for Directory '$new_base_path'" . PHP_EOL;
@@ -218,22 +224,22 @@ foreach ($data_sources as $info) {
 	 */
 	if (!file_exists($old_rrd_path)) {
 		/* check for file in other path */
-		$data_source_name = db_fetch_cell('SELECT data_source_name
+		$data_source_name = db_fetch_cell_prepared('SELECT data_source_name
 			FROM data_template_rrd
 			WHERE local_data_id = ?
 			LIMIT 1',
 			array($local_data_id));
 
-		$data_source_path1 = $base_path_rra . '/' . strtolower(clean_up_file_name($info['description'])) . '_' . $local_data_id . '.rrd';
+		$data_source_path1 = $base_rra_path . '/' . strtolower(clean_up_file_name($info['description'])) . '_' . $local_data_id . '.rrd';
 
 		if ($pattern == '' || $pattern == 'device') {
-			$data_source_path2 = $base_path_rra . '/' . $info['host_id'] . '/' . $info['snmp_query_id'] . '/' . $local_data_id . '.rrd';
+			$data_source_path2 = $base_rra_path . '/' . $info['host_id'] . '/' . $info['snmp_query_id'] . '/' . $local_data_id . '.rrd';
 		} elseif ($pattern == 'device_dq') {
-			$data_source_path2 = $base_path_rra . '/' . $info['host_id'] . '/' . $local_data_id . '.rrd';
+			$data_source_path2 = $base_rra_path . '/' . $info['host_id'] . '/' . $local_data_id . '.rrd';
 		} elseif ($pattern == 'hash_device') {
-			$data_source_path2 = $base_path_rra . '/' . $info['hash_id'] . '/' . $info['host_id'] . '/' . $local_data_id . '.rrd';
+			$data_source_path2 = $base_rra_path . '/' . $info['hash_id'] . '/' . $info['host_id'] . '/' . $local_data_id . '.rrd';
 		} elseif ($pattern == 'hash_device_dq') {
-			$data_source_path2 = $base_path_rra . '/' . $info['hash_id'] . '/' . $info['host_id'] . '/' . $info['snmp_query_id'] . '/' . $local_data_id . '.rrd';
+			$data_source_path2 = $base_rra_path . '/' . $info['hash_id'] . '/' . $info['host_id'] . '/' . $info['snmp_query_id'] . '/' . $local_data_id . '.rrd';
 		}
 
 		if (file_exists($data_source_path1)) {
@@ -262,7 +268,7 @@ foreach ($data_sources as $info) {
 				struct_debug("Move Completed for: '" . $old_rrd_path . "' > '" . $new_rrd_path . "'");
 
 				if ($config['cacti_server_os'] != 'win32') {
-					if (recursive_chown($new_rrd_path, $owner_id) && chgrp($new_rrd_path, $group_id)) {
+					if (sp_recursive_chown($new_rrd_path, $owner_id) && sp_recursive_chgrp($new_rrd_path, $group_id)) {
 						struct_debug("Permissions set for '$new_rrd_path'");
 					} else {
 						print "FATAL: Could not Set Permissions for File '$new_rrd_path'" . PHP_EOL;
@@ -332,20 +338,20 @@ function update_database($info) {
 }
 
 /**
- * recursive_chown - Recursively chown on a path
+ * sp_recursive_chown - Recursively chown on a path
  *
  * @param  (string)     $path
  * @param  (string|int) $user
  *
  * @return (void)
  */
-function recursive_chown($path, $user) {
+function sp_recursive_chown($path, $user) {
 	$directory = rtrim($path, '/');
 
 	if ($items = glob($path . '/*')) {
 		foreach ($items as $item) {
 			if (is_dir($item)) {
-				return recursive_chown($item, $user);
+				return sp_recursive_chown($item, $user);
 			} else {
 				return chown($item, $user);
 			}
@@ -356,20 +362,20 @@ function recursive_chown($path, $user) {
 }
 
 /**
- * recursive_chgrp - Recursively chgrp on a path
+ * sp_recursive_chgrp - Recursively chgrp on a path
  *
  * @param  (string)     $path
  * @param  (string|int) $group
  *
  * @return (void)
  */
-function recursive_chgrp($path, $group) {
+function sp_recursive_chgrp($path, $group) {
 	$directory = rtrim($path, '/');
 
 	if ($items = glob($path . '/*')) {
 		foreach ($items as $item) {
 			if (is_dir($item)) {
-				return recursive_chgrp($item, $group);
+				return sp_recursive_chgrp($item, $group);
 			} else {
 				return chgrp($item, $group);
 			}
