@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -28,6 +28,8 @@ include('./include/auth.php');
 include_once('./lib/html_tree.php');
 include_once('./lib/html_graph.php');
 include_once('./lib/api_tree.php');
+include_once('./lib/graphs.php');
+include_once('./lib/reports.php');
 include_once('./lib/timespan_settings.php');
 
 /* set the default graph action */
@@ -52,29 +54,54 @@ function get_matching_nodes() {
 		$matching = db_fetch_assoc_prepared("SELECT gti.parent, gti.graph_tree_id
 			FROM graph_tree_items AS gti
 			LEFT JOIN host AS h
-			ON h.id=gti.host_id
-			LEFT JOIN graph_templates_graph AS gtg
-			ON gtg.local_graph_id=gti.local_graph_id AND gtg.local_graph_id>0
-			WHERE gtg.title_cache LIKE ?
-			OR h.description LIKE ?
-			OR h.hostname LIKE ?
-			OR gti.title LIKE ?",
-			array($filter, $filter, $filter, $filter));
+			ON h.id = gti.host_id
+			LEFT JOIN (
+				SELECT DISTINCT site_id
+				FROM host
+				WHERE description LIKE ?
+				OR hostname LIKE ?
+			) AS h2
+			ON h2.site_id = gti.site_id
+			LEFT JOIN (
+				SELECT local_graph_id
+				FROM graph_templates_graph
+				WHERE local_graph_id > 0
+				AND title_cache LIKE ?
+			) AS gtg
+			ON gtg.local_graph_id = gti.local_graph_id
+			LEFT JOIN (
+				SELECT id
+				FROM sites
+				WHERE name LIKE ?
+			) AS site
+			ON site.id = gti.site_id
+			WHERE (gti.title LIKE ?)
+			OR (h.description LIKE ? AND (gti.host_id > 0 OR gti.site_id > 0))
+			OR (h.hostname LIKE ? AND (gti.host_id > 0 OR gti.site_id > 0))
+			OR (h2.site_id > 0)
+			OR (gtg.local_graph_id > 0)
+			OR (site.id > 0)",
+			array($filter, $filter, $filter, $filter, $filter, $filter, $filter));
 	} else {
 		$matching = db_fetch_assoc("SELECT parent, graph_tree_id FROM graph_tree_items");
 	}
 
-	if (sizeof($matching)) {
+	if (cacti_sizeof($matching)) {
 		foreach($matching as $row) {
 			while ($row['parent'] != '0') {
 				$match[] = 'tbranch-' . $row['parent'];
-				$row = db_fetch_row_prepared('SELECT parent, graph_tree_id FROM graph_tree_items WHERE id = ?', array($row['parent']));
-				if (!sizeof($row)) {
-					break;
+
+				$row = db_fetch_row_prepared('SELECT parent, graph_tree_id
+					FROM graph_tree_items
+					WHERE id = ?',
+					array($row['parent']));
+
+				if (!cacti_sizeof($row)) {
+				    break;
 				}
 			}
 
-			if (sizeof($row)) {
+			if (cacti_sizeof($row)) {
 				$match[]      = 'tree_anchor-' . $row['graph_tree_id'];
 				$my_matches[] = array_reverse($match);
 				$match        = array();
@@ -99,13 +126,13 @@ function get_matching_nodes() {
 			$level++;
 
 			if ($found == 0) {
-				break;
+			    break;
 			}
 		}
 
-		if (sizeof($final_array)) {
-			$fa = array();
+		$fa = array();
 
+		if (cacti_sizeof($final_array)) {
 			foreach($final_array as $key => $matches) {
 				foreach($matches as $branch => $dnc) {
 					$fa[] = $branch;
@@ -116,7 +143,6 @@ function get_matching_nodes() {
 		header('Content-Type: application/json; charset=utf-8');
 
 		print json_encode($fa);
-		//print '[' . implode(', ', array_keys($matching)) . ']';
 	}
 }
 
@@ -129,9 +155,41 @@ case 'ajax_search':
 	get_matching_nodes();
 	exit;
 
+	break;
+case 'ajax_reports':
+	// Add to a report
+	get_filter_request_var('report_id');
+	get_filter_request_var('timespan');
+	get_filter_request_var('align');
+
+	if (isset_request_var('graph_list')) {
+		$items = explode(',', get_request_var('graph_list'));
+
+		if (cacti_sizeof($items)) {
+			$good = true;
+
+			foreach($items as $item) {
+				if (!reports_add_graphs(get_filter_request_var('report_id'), $item, get_request_var('timespan'), get_request_var('align'))) {
+					raise_message('reports_add_error');
+					$good = false;
+					break;
+				}
+			}
+
+			if ($good) {
+				raise_message('reports_graphs_added');
+			}
+		}
+	} else {
+		raise_message('reports_no_graph');
+	}
+
+	header('Location: graph_view.php?action=list&header=false');
+
 	break;
 case 'update_timespan':
-	// we really don't need to do anything.  The session variables have already been updated
+	$_SESSION['sess_current_date1'] = get_request_var('date1');
+	$_SESSION['sess_current_date2'] = get_request_var('date2');
 
 	break;
 case 'save':
@@ -140,7 +198,6 @@ case 'save':
 		get_filter_request_var('predefined_timespan');
 		get_filter_request_var('predefined_timeshift');
 		get_filter_request_var('graphs');
-		get_filter_request_var('graph_template_id', FILTER_VALIDATE_IS_NUMERIC_LIST);
 		get_filter_request_var('thumbnails', FILTER_VALIDATE_REGEXP, array('options' => array('regexp' => '(true|false)')));
 
 		if (isset_request_var('predefined_timespan')) {
@@ -151,10 +208,6 @@ case 'save':
 			set_graph_config_option('default_timeshift', get_request_var('predefined_timeshift'));
 		}
 
-		if (isset_request_var('graph_template_id')) {
-			set_graph_config_option('graph_template_id', get_request_var('graph_template_id'));
-		}
-
 		if (isset_request_var('section') && get_nfilter_request_var('section') == 'preview') {
 			if (isset_request_var('columns')) {
 				set_graph_config_option('num_columns', get_request_var('columns'));
@@ -180,12 +233,21 @@ case 'save':
 
 	break;
 case 'tree':
+	html_validate_tree_vars();
+
 	if (isset_request_var('tree_id')) {
 		$_SESSION['sess_tree_id'] = get_filter_request_var('tree_id');
 	}
 
 	top_graph_header();
 
+	?>
+	<script type='text/javascript'>
+	minTreeWidth = <?php print read_user_setting('min_tree_width');?>;
+	maxTreeWidth = <?php print read_user_setting('max_tree_width');?>;
+	</script>
+	<?php
+
 	bottom_footer();
 
 	break;
@@ -197,7 +259,8 @@ case 'get_node':
 		if (get_nfilter_request_var('tree_id') == 0 && strstr(get_nfilter_request_var('id'), 'tbranch-') !== false) {
 			$tree_id = db_fetch_cell_prepared('SELECT graph_tree_id
 				FROM graph_tree_items
-				WHERE id = ?', array(str_replace('tbranch-', '', get_nfilter_request_var('id'))));
+				WHERE id = ?',
+				array(str_replace('tbranch-', '', get_nfilter_request_var('id'))));
 		}else if (get_nfilter_request_var('tree_id') == 'default' ||
 			get_nfilter_request_var('tree_id') == 'undefined' ||
 			get_nfilter_request_var('tree_id') == '') {
@@ -226,7 +289,12 @@ case 'get_node':
 				if ($pnode[0] == 'tbranch') {
 					$parent = $pnode[1];
 					input_validate_input_number($parent);
-					$tree_id = db_fetch_cell_prepared('SELECT graph_tree_id FROM graph_tree_items WHERE id = ?', array($parent));
+
+					$tree_id = db_fetch_cell_prepared('SELECT graph_tree_id
+						FROM graph_tree_items
+						WHERE id = ?',
+						array($parent));
+
 					break;
 				}
 			}
@@ -239,15 +307,18 @@ case 'get_node':
 case 'tree_content':
 	html_validate_tree_vars();
 
+	top_graph_header();
+
 	if (!is_view_allowed('show_tree')) {
-		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR TREE VIEW') . "</font>"; return;
+		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR TREE VIEW') . '</font>';
+		exit;
 	}
 
 	if (!isempty_request_var('node')) {
-		$_SESSION['sess_graph_node'] = get_request_var('node');
+		$_SESSION['sess_graph_node'] = sanitize_search_string(get_nfilter_request_var('node'));
 
 		if (!isempty_request_var('hgd')) {
-			$_SESSION['sess_graph_hgd'] = get_request_var('hgd');
+			$_SESSION['sess_graph_hgd'] = sanitize_search_string(get_nfilter_request_var('hgd'));
 		} else {
 			$_SESSION['sess_graph_hgd'] = '';
 		}
@@ -258,16 +329,16 @@ case 'tree_content':
 
 	?>
 	<script type='text/javascript'>
-	var refreshIsLogout=false;
-	var refreshPage='<?php print str_replace('tree_content', 'tree', $_SERVER['REQUEST_URI']);?>';
-	var refreshMSeconds=<?php print read_user_setting('page_refresh')*1000;?>;
-	var graph_start=<?php print get_current_graph_start();?>;
-	var graph_end=<?php print get_current_graph_end();?>;
-	var timeOffset=<?php print date('Z');?>
+	var refreshIsLogout = false;
+	var refreshPage     = '<?php print str_replace('tree_content', 'tree', sanitize_uri($_SERVER['REQUEST_URI']));?>';
+	var refreshMSeconds = <?php print read_user_setting('page_refresh')*1000;?>;
+	var graph_start     = <?php print get_current_graph_start();?>;
+	var graph_end       = <?php print get_current_graph_end();?>;
+	var timeOffset      = <?php print date('Z');?>
 
 	// Adjust the height of the tree
 	$(function() {
-		myGraphLocation='tree';
+		pageAction   = 'tree';
 		navHeight    = $('.cactiTreeNavigationArea').height();
 		windowHeight = $(window).height();
 		navOffset    = $('.cactiTreeNavigationArea').offset();
@@ -285,13 +356,13 @@ case 'tree_content':
 	$hgdata  = 0;
 
 	if (isset_request_var('node')) {
-		$parts = explode('-', get_request_var('node'));
+		$parts = explode('-', sanitize_search_string(get_request_var('node')));
 
-		// Check for tree anchoe
-		if (strpos(get_request_var('node'), 'tree_anchor') !== false) {
+		// Check for tree anchor
+		if (strpos(get_nfilter_request_var('node'), 'tree_anchor') !== false) {
 			$tree_id = $parts[1];
 			$node_id = 0;
-		} elseif (strpos(get_request_var('node'), 'tbranch') !== false) {
+		} elseif (strpos(get_nfilter_request_var('node'), 'tbranch') !== false) {
 			// Check for branch
 			$node_id = $parts[1];
 			$tree_id = db_fetch_cell_prepared('SELECT graph_tree_id
@@ -314,16 +385,28 @@ case 'tree_content':
 		grow_right_pane_tree($tree_id, $node_id, $hgdata);
 	}
 
+	bottom_footer();
+
 	break;
 case 'preview':
-	top_graph_header();
-
 	if (!is_view_allowed('show_preview')) {
-		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR PREVIEW VIEW') . "</font>"; return;
+		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR PREVIEW VIEW') . "</font>";
+		bottom_footer();
+		exit;
+	}
+
+	if (isset_request_var('external_id')) {
+		$host_id = db_fetch_cell_prepared('SELECT id FROM host WHERE external_id = ?', array(get_nfilter_request_var('external_id')));
+		if (!empty($host_id)) {
+			set_request_var('host_id', $host_id);
+			set_request_var('reset',true);
+		}
 	}
 
 	html_graph_validate_preview_request_vars();
 
+	top_graph_header();
+
 	/* include graph view filter selector */
 	html_start_box(__('Graph Preview Filters') . (isset_request_var('style') && get_request_var('style') != '' ? ' ' . __('[ Custom Graph List Applied - Filtering from List ]'):''), '100%', '', '3', 'center', '');
 
@@ -340,12 +423,16 @@ case 'preview':
 			/* process selected graphs */
 			if (!isempty_request_var('graph_list')) {
 				foreach (explode(',', get_request_var('graph_list')) as $item) {
-					$graph_list[$item] = 1;
+					if (is_numeric($item)) {
+						$graph_list[$item] = 1;
+					}
 				}
 			}
 			if (!isempty_request_var('graph_add')) {
 				foreach (explode(',', get_request_var('graph_add')) as $item) {
-					$graph_list[$item] = 1;
+					if (is_numeric($item)) {
+						$graph_list[$item] = 1;
+					}
 				}
 			}
 			/* remove items */
@@ -361,7 +448,7 @@ case 'preview':
 				$i++;
 			}
 
-			if ((isset($graph_array)) && (sizeof($graph_array) > 0)) {
+			if ((isset($graph_array)) && (cacti_sizeof($graph_array) > 0)) {
 				/* build sql string including each graph the user checked */
 				$sql_or = array_to_sql_or($graph_array, 'gtg.local_graph_id');
 			}
@@ -384,14 +471,16 @@ case 'preview':
 		$sql_where .= (empty($sql_where) ? '' : ' AND') . ' gl.host_id=0';
 	}
 
-	if (!isempty_request_var('graph_template_id')) {
-		$sql_where .= (empty($sql_where) ? '' : ' AND') . ' gl.graph_template_id IN (' . get_request_var('graph_template_id') . ')';
+	if (!isempty_request_var('graph_template_id') && get_request_var('graph_template_id') != '-1' && get_request_var('graph_template_id') != '0') {
+		$sql_where .= ($sql_where != '' ? ' AND ':'') . ' (gl.graph_template_id IN (' . get_request_var('graph_template_id') . '))';
+	} elseif (get_request_var('graph_template_id') == '0') {
+		$sql_where .= ($sql_where != '' ? ' AND ':'') . ' (gl.graph_template_id IN (' . get_request_var('graph_template_id') . '))';
 	}
 
-	$limit = (get_request_var('graphs') * (get_request_var('page') - 1)) . ',' . get_request_var('graphs');
-	$order = 'gtg.title_cache';
+	$limit      = (get_request_var('graphs')*(get_request_var('page')-1)) . ',' . get_request_var('graphs');
+	$order      = 'gtg.title_cache';
 
-	$graphs = get_allowed_graphs($sql_where, $order, $limit, $total_graphs);
+	$graphs     = get_allowed_graphs($sql_where, $order, $limit, $total_graphs);
 
 	$nav = html_nav_bar('graph_view.php', MAX_DISPLAY_PAGES, get_request_var('page'), get_request_var('graphs'), $total_graphs, get_request_var('columns'), __('Graphs'), 'page', 'main');
 
@@ -417,16 +506,17 @@ case 'preview':
 
 	break;
 case 'list':
-	top_graph_header();
+	global $graph_timespans, $alignment, $graph_sources;
 
 	if (!is_view_allowed('show_list')) {
-		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR LIST VIEW') . '</font>'; return;
+		print "<font class='txtErrorTextBox'>" . __('YOU DO NOT HAVE RIGHTS FOR LIST VIEW') . '</font>';
+		bottom_footer();
+		exit;
 	}
 
 	/* reset the graph list on a new viewing */
 	if (!isset_request_var('page')) {
 		set_request_var('graph_list', '');
-		set_request_var('page', 1);
 	}
 
 	/* ================= input validation and session storage ================= */
@@ -448,23 +538,23 @@ case 'list':
 		'graph_template_id' => array(
 			'filter' => FILTER_VALIDATE_IS_NUMERIC_LIST,
 			'pageset' => true,
-			'default' => '0'
+			'default' => '-1'
 			),
 		'host_id' => array(
 			'filter' => FILTER_VALIDATE_INT,
 			'pageset' => true,
 			'default' => '-1'
 			),
-		'graph_list' => array(
-			'filter' => FILTER_DEFAULT,
+		'graph_add' => array(
+			'filter' => FILTER_VALIDATE_IS_NUMERIC_LIST,
 			'default' => ''
 			),
-		'graph_add' => array(
-			'filter' => FILTER_DEFAULT,
+		'graph_list' => array(
+			'filter' => FILTER_VALIDATE_IS_NUMERIC_LIST,
 			'default' => ''
 			),
 		'graph_remove' => array(
-			'filter' => FILTER_DEFAULT,
+			'filter' => FILTER_VALIDATE_IS_NUMERIC_LIST,
 			'default' => ''
 			)
 	);
@@ -483,13 +573,17 @@ case 'list':
 	/* save selected graphs into url */
 	if (!isempty_request_var('graph_list')) {
 		foreach (explode(',', get_request_var('graph_list')) as $item) {
-			$graph_list[$item] = 1;
+			if (is_numeric($item)) {
+				$graph_list[$item] = 1;
+			}
 		}
 	}
 
 	if (!isempty_request_var('graph_add')) {
 		foreach (explode(',', get_request_var('graph_add')) as $item) {
-			$graph_list[$item] = 1;
+			if (is_numeric($item)) {
+				$graph_list[$item] = 1;
+			}
 		}
 	}
 
@@ -501,12 +595,19 @@ case 'list':
 	}
 
 	/* update the revised graph list session variable */
-	if (sizeof($graph_list)) {
+	if (cacti_sizeof($graph_list)) {
 		set_request_var('graph_list', implode(',', array_keys($graph_list)));
 	}
 	load_current_session_value('graph_list', 'sess_gl_graph_list', '');
 
-	form_start('graph_view.php', 'form_graph_list');
+	$reports = db_fetch_assoc_prepared('SELECT *
+		FROM reports
+		WHERE user_id = ?',
+		array($_SESSION['sess_user_id']));
+
+	top_graph_header();
+
+	form_start('graph_view.php', 'chk');
 
 	/* display graph view filter selector */
 	html_start_box(__('Graph List View Filters') . (isset_request_var('style') && get_request_var('style') != '' ? ' ' . __('[ Custom Graph List Applied - Filter FROM List ]'):''), '100%', '', '3', 'center', '');
@@ -520,19 +621,37 @@ case 'list':
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input id='rfilter' type='text' size='30' value='<?php print html_escape_request_var('rfilter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='rfilter' size='55' value='<?php print html_escape_request_var('rfilter');?>'>
 					</td>
 					<?php html_host_filter(get_request_var('host_id'));?>
+					<td>
+						<span>
+							<input type='submit' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>' onClick='clearFilter()'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' value='<?php print __esc('View');?>' title='<?php print __esc('View Graphs');?>' onClick='viewGraphs()'>
+							<?php if (cacti_sizeof($reports)) {?>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='addreport' value='<?php print __esc('Report');?>' title='<?php print __esc('Add to a Report');?>'>
+							<?php } ?>
+						</span>
+					</td>
+				</tr>
+			</table>
+			<table class='filterTable'>
+				<tr>
 					<td>
 						<?php print __('Template');?>
 					</td>
 					<td>
 						<select id='graph_template_id' multiple style='opacity:0.1;overflow:hide;height:0px;'>
-							<option value='0'<?php if (get_request_var('graph_template_id') == '0') {?> selected<?php }?>><?php print __('All Graphs & Templates');?></option>
+							<option value='-1'<?php if (get_request_var('graph_template_id') == '-1') {?> selected<?php }?>><?php print __('All Graphs & Templates');?></option>
+							<option value='0'<?php if (get_request_var('graph_template_id') == '0') {?> selected<?php }?>><?php print __('Not Templated');?></option>
 							<?php
 
-							$graph_templates = get_allowed_graph_templates();
-							if (sizeof($graph_templates)) {
+							// suppress total rows collection
+							$total_rows = -1;
+
+							$graph_templates = get_allowed_graph_templates('', 'name', '', $total_rows);
+							if (cacti_sizeof($graph_templates)) {
 								$selected    = explode(',', get_request_var('graph_template_id'));
 								foreach ($graph_templates as $gt) {
 									if ($gt['id'] != 0) {
@@ -543,13 +662,13 @@ case 'list':
 
 										if ($found) {
 											print "<option value='" . $gt['id'] . "'";
-											if (sizeof($selected)) {
+											if (cacti_sizeof($selected)) {
 												if (in_array($gt['id'], $selected)) {
 													print ' selected';
 												}
 											}
 											print '>';
-											print $gt['name'] . "</option>\n";
+											print html_escape($gt['name']) . "</option>\n";
 										}
 									}
 								}
@@ -564,7 +683,7 @@ case 'list':
 						<select id='rows' onChange='applyFilter()'>
 							<option value='-1'<?php print (get_request_var('rows') == '-1' ? ' selected>':'>') . __('Default');?></option>
 							<?php
-							if (sizeof($item_rows)) {
+							if (cacti_sizeof($item_rows)) {
 								foreach ($item_rows as $key => $value) {
 									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . $value . "</option>\n";
 								}
@@ -572,13 +691,6 @@ case 'list':
 							?>
 						</select>
 					</td>
-					<td>
-						<span>
-							<input type='submit' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
-							<input type='button' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>' onClick='clearFilter()'>
-							<input type='button' value='<?php print __esc('View');?>' title='<?php print __esc('View Graphs');?>' onClick='viewGraphs()'>
-						</span>
-					</td>
 				</tr>
 			</table>
 			<input type='hidden' id='style' value='selective'>
@@ -603,8 +715,10 @@ case 'list':
 		$sql_where .= ($sql_where == '' ? '' : ' AND') . ' gl.host_id=0';
 	}
 
-	if (!isempty_request_var('graph_template_id')) {
-		$sql_where .= ($sql_where == '' ? '' : ' AND') . ' gl.graph_template_id IN (' . get_request_var('graph_template_id') . ')';
+	if (!isempty_request_var('graph_template_id') && get_request_var('graph_template_id') != '-1' && get_request_var('graph_template_id') != '0') {
+		$sql_where .= ($sql_where != '' ? ' AND ':'') . ' (gl.graph_template_id IN (' . get_request_var('graph_template_id') . '))';
+	} elseif (get_request_var('graph_template_id') == '0') {
+		$sql_where .= ($sql_where != '' ? ' AND ':'') . ' (gl.graph_template_id IN (' . get_request_var('graph_template_id') . '))';
 	}
 
 	$total_rows = 0;
@@ -618,54 +732,129 @@ case 'list':
 
 	html_start_box('', '100%', '', '3', 'center', '');
 
-	html_header_checkbox(array(__('Graph Title'), __('Device'), __('Graph Template'), __('Graph Size')), false);
+	if (is_realm_allowed(10)) {
+		$display_text = array(
+			'title_cache' => array(
+				'display' => __('Graph Name'),
+				'align'   => 'left',
+				'tip'     => __('The Title of this Graph.  Generally programmatically generated from the Graph Template definition or Suggested Naming rules.  The max length of the Title is controlled under Settings->Visual.')
+			),
+			'local_graph_id' => array(
+				'display' => __('Device'),
+				'align'   => 'left',
+				'tip'     => __('The device for this Graph.')
+			),
+			'source' => array(
+				'display' => __('Source Type'),
+				'align'   => 'right',
+				'tip'     => __('The underlying source that this Graph was based upon.')
+			),
+			'name' => array(
+				'display' => __('Source Name'),
+				'align'   => 'left',
+				'tip'     => __('The Graph Template or Data Query that this Graph was based upon.')
+			),
+			'height' => array(
+				'display' => __('Size'),
+				'align'   => 'left',
+				'tip'     => __('The size of this Graph when not in Preview mode.')
+			)
+		);
+	} else {
+		$display_text = array(
+			'title_cache' => array(
+				'display' => __('Graph Name'),
+				'align'   => 'left',
+				'tip'     => __('The Title of this Graph.  Generally programmatically generated from the Graph Template definition or Suggested Naming rules.  The max length of the Title is controlled under Settings->Visual.')
+			),
+			'height' => array(
+				'display' => __('Size'),
+				'align'   => 'left',
+				'tip'     => __('The size of this Graph when not in Preview mode.')
+			)
+		);
+	}
+
+	html_header_checkbox($display_text, false);
 
 	$i = 0;
-	if (sizeof($graphs)) {
+	if (cacti_sizeof($graphs)) {
 		foreach ($graphs as $graph) {
-			if ($graph['description'] == '' && $graph['template_name'] == '') {
-				$aggregate = db_fetch_cell_prepared('SELECT agt.name
-					FROM aggregate_graphs AS ag
-					INNER JOIN aggregate_graph_templates AS agt
-					ON ag.aggregate_template_id=agt.id
-					WHERE local_graph_id = ?',
-					array($graph['local_graph_id']));
+			/* we're escaping strings here, so no need to escape them on form_selectable_cell */
+			$template_details = get_graph_template_details($graph['local_graph_id']);
 
-				if (!empty($aggregate)) {
-					$graph['description']   = __('Aggregated Device');
-					$graph['template_name'] = $aggregate;
-				} else {
-					$graph['description']   = __('Non-Device');
-					$graph['template_name'] = __('Not Applicable');
-				}
+			if($graph['graph_source'] == '0') { //Not Templated, customize graph source and template details.
+				$template_details = api_plugin_hook_function('customize_template_details', $template_details);
+				$graph = api_plugin_hook_function('customize_graph', $graph);
+			}
+
+			if (isset($template_details['graph_name'])) {
+				$graph['name'] = $template_details['graph_name'];
+			}
+
+			if (isset($template_details['graph_description'])) {
+				$graph['description'] = $template_details['graph_description'];
 			}
 
 			form_alternate_row('line' . $graph['local_graph_id'], true);
 			form_selectable_cell(filter_value($graph['title_cache'], get_request_var('rfilter'), 'graph.php?local_graph_id=' . $graph['local_graph_id'] . '&rra_id=0'), $graph['local_graph_id']);
-			form_selectable_cell($graph['description'], $graph['local_graph_id']);
-			form_selectable_cell($graph['template_name'], $graph['local_graph_id']);
-			form_selectable_cell($graph['height'] . 'x' . $graph['width'], $graph['local_graph_id']);
+			if (is_realm_allowed(10)) {
+				form_selectable_ecell($graph['description'], $graph['local_graph_id']);
+				form_selectable_cell(filter_value($graph_sources[$template_details['source']], get_request_var('rfilter')), $graph['local_graph_id'], '', 'right');
+				form_selectable_cell(filter_value($template_details['name'], get_request_var('rfilter'), $template_details['url']), $graph['local_graph_id'], '', 'left');
+			}
+			form_selectable_ecell($graph['height'] . 'x' . $graph['width'], $graph['local_graph_id']);
 			form_checkbox_cell($graph['title_cache'], $graph['local_graph_id']);
 			form_end_row();
 		}
 	}
 
-	?>
-	<tr>
-		<td style='text-align:right' colspan='3'><img src='images/arrow.gif' alt=''>&nbsp;</td>
-		<td style='text-align:right' colspan='2'><input type='button' value='<?php print __esc('View');?>' title='<?php print __esc('View Graphs');?>' onClick='viewGraphs()'></td>
-	</tr>
-	<?php
-
 	html_end_box(false);
 
-	if (sizeof($graphs)) {
+	if (cacti_sizeof($graphs)) {
 		print $nav;
 	}
 
 	form_end();
 
+	$report_text = '';
+
+	if (cacti_sizeof($reports)) {
+		$report_text = '<div id="addGraphs" style="display:none;">
+			<p>' . __('Select the Report to add the selected Graphs to.') . '</p>
+			<table class="cactiTable">';
+
+		$report_text .= '<tr><td>' . __('Report Name') . '</td>';
+		$report_text .= '<td><select id="report_id">';
+		foreach($reports as $report) {
+			$report_text .= '<option value="' . $report['id'] . '">' . html_escape($report['name']) . '</option>';
+		}
+		$report_text .= '</select></td></tr>';
+
+		$report_text .= '<tr><td>' . __('Timespan') . '</td>';
+		$report_text .= '<td><select id="timespan">';
+		foreach($graph_timespans as $key => $value) {
+			$report_text .= '<option value="' . $key . '"' . ($key == read_user_setting('default_timespan') ? ' selected':'') . '>' . $value . '</option>';
+		}
+		$report_text .= '</select></td></tr>';
+
+		$report_text .= '<tr><td>' . __('Align') . '</td>';
+		$report_text .= '<td><select id="align">';
+		foreach($alignment as $key => $value) {
+			$report_text .= '<option value="' . $key . '"' . ($key == REPORTS_ALIGN_CENTER ? ' selected':'') . '>' . $value . '</option>';
+		}
+		$report_text .= '</select></td></tr>';
+
+		$report_text .= '</table></div>';
+	}
+
 	?>
+	<div class='break'></div>
+	<div class='cactiTable'>
+		<div style='float:left'><img src='images/arrow.gif' alt=''>&nbsp;</div>
+		<div style='float:right'><input type='button' class='ui-button ui-corner-all ui-widget' value='<?php print __esc('View');?>' title='<?php print __esc('View Graphs');?>' onClick='viewGraphs()'></div>
+	</div>
+	<?php print $report_text;?>
 	<script type='text/javascript'>
 	var refreshMSeconds=999999999;
 	var graph_list_array = new Array(<?php print get_request_var('graph_list');?>);
@@ -704,22 +893,22 @@ case 'list':
 		$('#graph_list').val(graphList);
 
 		strURL = urlPath+'graph_view.php?action=preview';
-		$('#form_graph_list').find('select, input').each(function() {
+		$('#chk').find('select, input').each(function() {
 			switch($(this).attr('id')) {
-			case 'graph_template_id':
-				strURL += '&' + $(this).attr('id') + '=' + $(this).val();
-				break;
-			case 'host_id':
-			case 'rfilter':
-			case 'graph_add':
-			case 'graph_remove':
-			case 'graph_list':
-			case 'style':
-			case 'csrf_magic':
-				strURL += '&' + $(this).attr('id') + '=' + $(this).val();
-				break;
-			default:
-				break;
+				case 'rfilter':
+					strURL += '&' + $(this).attr('id') + '=' + base64_encode($(this).val());
+					break;
+				case 'graph_template_id':
+				case 'host_id':
+				case 'graph_add':
+				case 'graph_remove':
+				case 'graph_list':
+				case 'style':
+				case 'csrf_magic':
+					strURL += '&' + $(this).attr('id') + '=' + $(this).val();
+					break;
+				default:
+					break;
 			}
 		});
 
@@ -764,74 +953,72 @@ case 'list':
 		return false;
 	}
 
+	function addReport() {
+		$('#addGraphs').dialog({
+			title: '<?php print __('Add Selected Graphs to Report');?>',
+			minHeight: 80,
+			minWidth: 400,
+			modal: true,
+			resizable: false,
+			draggable: false,
+			buttons: [
+				{
+					text: '<?php print __('Cancel');?>',
+					click: function() {
+						$(this).dialog('close');
+					}
+				},
+				{
+					text: '<?php print __('Ok');?>',
+					click: function() {
+						graphList = $('#graph_list').val();
+						$('input[id^=chk_]').each(function(data) {
+							graphID = $(this).attr('id').replace('chk_','');
+							if ($(this).is(':checked')) {
+								graphList += (graphList.length > 0 ? ',':'') + graphID;
+							}
+						});
+						$('#graph_list').val(graphList);
+
+						$(this).dialog('close');
+
+						strURL = 'graph_view.php?action=ajax_reports';
+							'&header=false' +
+							'&report_id='   + $('#report_id').val()  +
+							'&timespan='    + $('#timespan').val()   +
+							'&align='       + $('#align').val()      +
+							'&graph_list='  + $('#graph_list').val();
+
+						loadPageNoHeader(strURL);
+					}
+				}
+			],
+			open: function() {
+				$('.ui-dialog').css('z-index', 99);
+				$('.ui-widget-overlay').css('z-index', 98);
+			},
+			close: function() {
+				$('[title]').each(function() {
+					if ($(this).tooltip('instance')) {
+						$(this).tooltip('close');
+					}
+				});
+			}
+		});
+	}
+
 	$(function() {
-		myGraphLocation='list';
+		pageAction = 'list';
 
 		initializeChecks();
 
-		var msWidth = 100;
-		$('#graph_template_id option').each(function() {
-			if ($(this).textWidth() > msWidth) {
-				msWidth = $(this).textWidth();
-			}
-			$('#graph_template_id').css('width', msWidth+120+'px');
+		$('#addreport').click(function() {
+			addReport();
 		});
 
-		$('#graph_template_id').hide().multiselect({
-			height: 300,
-			noneSelectedText: '<?php print __('All Graphs & Templates');?>',
-			selectedText: function(numChecked, numTotal, checkedItems) {
-				myReturn = numChecked + ' <?php print __('Templates Selected');?>';
-				$.each(checkedItems, function(index, value) {
-					if (value.value == '0') {
-						myReturn='<?php print __('All Graphs & Templates');?>';
-						return false;
-					}
-				});
-				return myReturn;
-			},
-			checkAllText: '<?php print __('All');?>',
-			uncheckAllText: '<?php print __('None');?>',
-			uncheckall: function() {
-				$(this).multiselect('widget').find(':checkbox:first').each(function() {
-					$(this).prop('checked', true);
-				});
-			},
-			open: function(event, ui) {
-				$("input[type='search']:first").focus();
-			},
-			close: function(event, ui) {
-				applyFilter();
-			},
-			click: function(event, ui) {
-				checked=$(this).multiselect('widget').find('input:checked').length;
+		<?php html_graph_template_multiselect('list');?>
 
-				if (ui.value == 0) {
-					if (ui.checked == true) {
-						$('#graph_template_id').multiselect('uncheckAll');
-						$(this).multiselect('widget').find(':checkbox:first').each(function() {
-							$(this).prop('checked', true);
-						});
-					}
-				}else if (checked == 0) {
-					$(this).multiselect('widget').find(':checkbox:first').each(function() {
-						$(this).click();
-					});
-				}else if ($(this).multiselect('widget').find('input:checked:first').val() == '0') {
-					if (checked > 0) {
-						$(this).multiselect('widget').find(':checkbox:first').each(function() {
-							$(this).click();
-							$(this).prop('disable', true);
-						});
-					}
-				}
-			}
-		}).multiselectfilter({
-			label: '<?php print __('Search');?>',
-			width: msWidth
-		});
-
-		$('#form_graph_list').unbind().on('submit', function(event) {
+		$('#chk').unbind().on('submit', function(event) {
 			event.preventDefault();
 			applyFilter();
 		});
