@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -134,6 +134,11 @@ $cacti_version_codes = array(
 	'1.2.19'  => '0103',
 	'1.2.20'  => '0103',
 	'1.2.21'  => '0103',
+	'1.2.22'  => '0103',
+	'1.2.23'  => '0103',
+	'1.2.24'  => '0103',
+	'1.2.25'  => '0103',
+	'1.2.26'  => '0103',
 );
 
 $messages = array(
@@ -273,13 +278,13 @@ $messages = array(
 		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before creating Graphs or Data Sources.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_save_w_ds' => array(
-		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input Input Fields.'),
+		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input - Input Fields.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_field_save_wo_ds' => array(
 		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before creating Graphs or Data Sources.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_field_save_w_ds' => array(
-		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input Input Fields.'),
+		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input - Input Fields.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'clog_invalid' => array(
 		'message' => __('Log file specified is not a Cacti log or archive file.'),
@@ -422,7 +427,58 @@ $cdef_functions = array(1 =>
 	'NEGINF',
 	'NOW',
 	'TIME',
-	'LTIME'
+	'LTIME',
+	'ADDNAN',
+	'TREND',
+	'TRENDNAN',
+	'PREDICT',
+	'PREDICTSIGMA',
+	'PREDICTPERC',
+	'SQRT',
+	'ATAN',
+	'ATAN2',
+	'POW',
+	'ISINF',
+	'MINNAN',
+	'MAXNAN',
+	'DEG2RAD',
+	'RAD2DEG',
+	'ABS',
+	'REV',
+	'SMIN',
+	'SMAX',
+	'MEDIAN',
+	'STDEV',
+	'PERCENT',
+	'COUNT',
+	'STEPWIDTH',
+	'NEWDAY',
+	'NEWWEEK',
+	'NEWMONTH',
+	'NEWYEAR',
+	'DEPTH',
+	'COPY',
+	'INDEX',
+	'ROLL'
+);
+
+$phperrors = array (
+	E_ERROR => 'ERROR',
+	E_WARNING => 'WARNING',
+	E_PARSE => 'PARSE',
+	E_NOTICE => 'NOTICE',
+	E_CORE_ERROR => 'CORE_ERROR',
+	E_CORE_WARNING => 'CORE_WARNING',
+	E_COMPILE_ERROR  => 'COMPILE_ERROR',
+	E_COMPILE_WARNING => 'COMPILE_WARNING',
+	E_USER_ERROR => 'USER_ERROR',
+	E_USER_WARNING => 'USER_WARNING',
+	E_USER_NOTICE  => 'USER_NOTICE',
+	E_STRICT => 'STRICT',
+	E_RECOVERABLE_ERROR  => 'RECOVERABLE_ERROR',
+	E_DEPRECATED => 'DEPRECATED',
+	E_USER_DEPRECATED  => 'USER_DEPRECATED',
+	E_ALL => 'ALL'
 );
 
 if (cacti_version_compare(get_rrdtool_version(), '1.8.0', '>=')) {
@@ -645,7 +701,7 @@ $poller_options = array(
 	2 => 'spine'
 );
 
-if (!file_exists(read_config_option('path_spine')) || !is_executable(read_config_option('path_spine'))) {
+if (read_config_option('path_spine') != '' && (!file_exists(read_config_option('path_spine')) || !is_executable(read_config_option('path_spine')))) {
 	unset($poller_options[2]);
 }
 
@@ -695,9 +751,10 @@ $heartbeats = array(
 	90     => __('%d Seconds', 90),
 	120    => __('%d Minutes', 2),
 	300    => __('%d Minutes', 5),
-	330    => __('%3.1f Minutes', 5.5),
+	330    => __('%0.1f Minutes', 5.5),
 	600    => __('%d Minutes', 10),
 	660    => __('%d Minutes', 11),
+	900    => __('%d Minutes', 15),
 	1200   => __('%d Minutes', 20),
 	1800   => __('%d Minutes', 30),
 	2400   => __('%d Minutes', 40),
@@ -772,6 +829,22 @@ $device_threads = array(
 	10 => __('%d Threads', 10)
 );
 
+$device_change_fields = array(
+	'preg_field' => array(
+		'/^snmp_/',
+		'/^ping_/'
+	),
+	'match_field' => array(
+		'poller_id',
+		'site_id',
+		'host_template_id',
+		'availability_method',
+		'device_threads',
+		'location',
+		'max_oids'
+	)
+);
+
 $cron_intervals = array(
 	60  => __('Every Minute'),
 	300 => __('Every %d Minutes', 5)
@@ -909,11 +982,11 @@ if ($config['poller_id'] == 1 || $config['connection'] == 'online') {
 	$menu = array(
 		__('Main Console') => array(
 			'index.php' => __('Console Page')
-			),
+		),
 		__('Create') => array(
 			'graphs_new.php' => __('New Graphs'),
 			'host.php?action=edit&create=true' => __('New Device')
-			),
+		),
 		__('Management') => array(
 			'host.php'             => __('Devices'),
 			'sites.php'            => __('Sites'),
@@ -921,48 +994,49 @@ if ($config['poller_id'] == 1 || $config['connection'] == 'online') {
 			'graphs.php'           => __('Graphs'),
 			'data_sources.php'     => __('Data Sources'),
 			'aggregate_graphs.php' => __('Aggregates'),
-			),
+		),
 		__('Data Collection') => array(
 			'pollers.php' => __('Data Collectors'),
 			'data_queries.php' => __('Data Queries'),
 			'data_input.php'   => __('Data Input Methods')
-			),
+		),
 		__('Templates') => array(
 			'host_templates.php'      => __('Device'),
 			'graph_templates.php'     => __('Graph'),
 			'data_templates.php'      => __('Data Source'),
 			'aggregate_templates.php' => __('Aggregate'),
 			'color_templates.php'     => __('Color')
-			),
+		),
 		__('Automation') => array(
 			'automation_networks.php'    => __('Networks'),
 			'automation_devices.php'     => __('Discovered Devices'),
 			'automation_templates.php'   => __('Device Rules'),
 			'automation_graph_rules.php' => __('Graph Rules'),
 			'automation_tree_rules.php'  => __('Tree Rules'),
-			'automation_snmp.php'        => __('SNMP Options'),
-			),
+		),
 		__('Presets') => array(
 			'data_source_profiles.php' => __('Data Profiles'),
+			'automation_snmp.php'      => __('SNMP'),
 			'cdef.php'                 => __('CDEFs'),
 			'vdef.php'                 => __('VDEFs'),
 			'color.php'                => __('Colors'),
-			'gprint_presets.php'       => __('GPRINTs')
-			),
+			'gprint_presets.php'       => __('GPRINTs'),
+		),
 		__('Import/Export') => array(
 			'templates_import.php' => __('Import Templates'),
+			'package_import.php'   => __('Import Packages'),
 			'templates_export.php' => __('Export Templates')
-			),
+		),
 		__('Configuration')  => array(
 			'settings.php'         => __('Settings'),
 			'user_admin.php'       => __('Users'),
 			'user_group_admin.php' => __('User Groups'),
 			'user_domains.php'     => __('User Domains')
-			),
+		),
 		__('Utilities') => array(
 			'utilities.php'  => __('System Utilities'),
 			'links.php'      => __('External Links'),
-			),
+		),
 		__('Troubleshooting') => array(
 			'data_debug.php' => __('Data Sources'),
 		)
@@ -1000,6 +1074,33 @@ $menu_glyphs = array(
 	__('Troubleshooting') => 'fa fa-bug'
 );
 
+$device_classes = array(
+	'wireless'     => __('Access Points, Controllers'),
+	'application'  => __('Application Related'),
+	'cacti'        => __('Cacti Related'),
+	'database'     => __('Database Related'),
+	'facilities'   => __('Facilities Related'),
+	'general'      => __('Generic Device'),
+	'hpc'          => __('HPC/Grid Computing'),
+	'hypervisor'   => __('Hypervisor Related'),
+	'remotemgmt'   => __('ILO, IPMI, iDrac, etc.'),
+	'license'      => __('Licensing Related'),
+	'linux'        => __('Linux Related'),
+	'loadbalancer' => __('Load Balancer'),
+	'switch'       => __('Network Switch'),
+	'router'       => __('Network Router'),
+	'nassan'       => __('NAS/SAN Related'),
+	'firewall'     => __('Network Firewalls'),
+	'power'        => __('Power Related PDUs/STS, etc.'),
+	'printer'      => __('Printer Related'),
+	'storage'      => __('Storage Related'),
+	'telephony'    => __('Telco Related'),
+	'webserver'    => __('Web Server Related'),
+	'windows'      => __('Windows Related'),
+	'ups'          => __('UPS Related'),
+	'unassigned'   => __('Unassigned')
+);
+
 if ((isset($_SESSION['sess_user_id']))) {
 	if (db_table_exists('external_links')) {
 		$consoles = db_fetch_assoc('SELECT id, title, extendedstyle
@@ -1189,11 +1290,13 @@ $user_auth_realm_filenames = array(
 	'step_json.php' => 26,
 	'managers.php' => 15,
 	'rrdcleaner.php' => 15,
+	'rrdcheck.php' => 15,
 	'settings.php' => 15,
 	'links.php' => 15,
 	'data_queries.php' => 13,
 	'templates_export.php' => 16,
 	'templates_import.php' => 17,
+	'package_import.php' => 17,
 	'tree.php' => 4,
 	'user_admin.php' => 1,
 	'user_domains.php' => 1,
@@ -1219,9 +1322,9 @@ $user_auth_realm_filenames = array(
 	'color_templates_items.php' => 5,
 	'aggregate_templates.php' => 5,
 	'aggregate_graphs.php' => 5,
-	'aggregate_items.php' => 5,
 	'spikekill.php' => 1043,
-	'permission_denied.php' => -1
+	'permission_denied.php' => -1,
+	'help.php' => -1
 );
 
 $hash_type_codes = array(
@@ -1447,6 +1550,13 @@ $dsstats_hourly_avg = array(
 	'360' => __('%d Hours', 6)
 );
 
+$rrdcheck_intervals = array(
+	'boost' => __('After Boost'),
+	'60'  => __('1 Hour'),
+	'240' => __('%d Hours', 4),
+	'1440' => __('%d Hours', 24)
+);
+
 $boost_max_rows_per_select = array(
 	'2000'   => __('2,000 Data Source Items'),
 	'5000'   => __('5,000 Data Source Items'),
@@ -1496,7 +1606,7 @@ $boost_max_memory = array(
 	'5120' => __('%d GBytes', 5),
 	'6144' => __('%d GBytes', 6),
 	'8192' => __('%d GBytes', 8),
-	'-1' => __('Infinity')
+	'-1' => __('Unlimited')
 );
 
 $realtime_window = array(
@@ -1781,7 +1891,7 @@ $i18n_months = array(
 	'January'	=> __('January'),
 	'February'	=> __('February'),
 	'March'		=> __('March'),
-	'Arpil'		=> __('April'),
+	'April'		=> __('April'),
 	'May'		=> __('May'),
 	'June'		=> __('June'),
 	'July'		=> __('July'),
@@ -1880,27 +1990,9 @@ $i18n_themes = array(
 $database_statuses = array(
 	0 => __('[Fail]'),
 	1 => __('[Warning]'),
-	2 => __('[Success]'),
-	3 => __('[Skipped]'),
-);
-
-$phperrors = array (
-	E_ERROR => 'ERROR',
-	E_WARNING => 'WARNING',
-	E_PARSE => 'PARSE',
-	E_NOTICE => 'NOTICE',
-	E_CORE_ERROR => 'CORE_ERROR',
-	E_CORE_WARNING => 'CORE_WARNING',
-	E_COMPILE_ERROR  => 'COMPILE_ERROR',
-	E_COMPILE_WARNING => 'COMPILE_WARNING',
-	E_USER_ERROR => 'USER_ERROR',
-	E_USER_WARNING => 'USER_WARNING',
-	E_USER_NOTICE  => 'USER_NOTICE',
-	E_STRICT => 'STRICT',
-	E_RECOVERABLE_ERROR  => 'RECOVERABLE_ERROR',
-	E_DEPRECATED => 'DEPRECATED',
-	E_USER_DEPRECATED  => 'USER_DEPRECATED',
-	E_ALL => 'ALL'
+	2 => __('[Restart]'),
+	3 => __('[Success]'),
+	4 => __('[Skipped]'),
 );
 
 $navigation = array(
@@ -2350,6 +2442,12 @@ $navigation = array(
 		'url' => 'rrdcleaner.php?action=restart',
 		'level' => '2'
 	),
+	'rrdcheck.php:' => array(
+		'title' => __('RRD Check'),
+		'mapping' => 'index.php:,utilities.php:',
+		'url' => 'rrdcheck.php',
+		'level' => '2'
+	),
 	'utilities.php:' => array(
 		'title' => __('Utilities'),
 		'mapping' => 'index.php:',
@@ -2530,6 +2628,12 @@ $navigation = array(
 		'url' => 'about.php',
 		'level' => '1'
 	),
+	'help.php:' => array(
+		'title' => __('Cacti Help'),
+		'mapping' => '',
+		'url' => 'help.php',
+		'level' => '1'
+	),
 	'templates_export.php:' => array(
 		'title' => __('Export Templates'),
 		'mapping' => 'index.php:',
@@ -2548,6 +2652,12 @@ $navigation = array(
 		'url' => 'templates_import.php',
 		'level' => '1'
 	),
+	'package_import.php:' => array(
+		'title' => __('Import Packages'),
+		'mapping' => 'index.php:',
+		'url' => 'package_import.php',
+		'level' => '1'
+	),
 	'reports_admin.php:' => array(
 		'title' => __('Reporting'),
 		'mapping' => '',
@@ -2668,24 +2778,6 @@ $navigation = array(
 		'url' => '',
 		'level' => '2'
 	),
-	'aggregate_items.php:' => array(
-		'title' => __('Aggregate Items'),
-		'mapping' => 'index.php:',
-		'url' => 'aggregate_items.php',
-		'level' => '1'
-	),
-	'aggregate_items.php:item_edit' => array(
-		'title' => __('(Edit)'),
-		'mapping' => 'index.php:,aggregate_graphs.php:,aggregate_items.php:',
-		'url' => '',
-		'level' => '2'
-	),
-	'aggregate_items.php:actions' => array(
-		'title' => __('Actions'),
-		'mapping' => 'index.php:,aggregate_items.php:',
-		'url' => '',
-		'level' => '2'
-	),
 	'automation_graph_rules.php:' => array(
 		'title' => __('Graph Rules'),
 		'mapping' => 'index.php:',
@@ -2815,4 +2907,20 @@ if ($config['cacti_server_os'] == 'unix') {
 	);
 }
 
+$allowed_proxy_headers =	array(
+	'X-Forwarded-For',
+	'X-Client-IP',
+	'X-Real-IP',
+	'X-ProxyUser-Ip',
+	'CF-Connecting-IP',
+	'True-Client-IP',
+	'HTTP_X_FORWARDED',
+	'HTTP_X_FORWARDED_FOR',
+	'HTTP_X_CLUSTER_CLIENT_IP',
+	'HTTP_FORWARDED_FOR',
+	'HTTP_FORWARDED',
+	'HTTP_CLIENT_IP',
+	'REMOTE_ADDR',
+);
+
 api_plugin_hook('config_arrays');
