@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -464,17 +464,36 @@ function find_best_path($binary_name) {
 function install_setup_get_templates() {
 	global $config;
 
-	@ini_set('zlib.output_compression', '0');
+	if ($config['is_web']) {
+		ini_set('zlib.output_compression', '0');
+	}
 
 	$templates = array(
+		'ACME.xml.gz',
+		'AKCP_Device.xml.gz',
+		'APC_InfraStruXure_InRow_CRAC.xml.gz',
+		'APC_InfraStruXure_PDU.xml.gz',
 		'Apache_Webserver.xml.gz',
+		'ArubaOS_switch.xml.gz',
+		'Aruba_OSCX_switch_6x00.xml.gz',
+		'Aruba_Instant_AP_Cluster.xml.gz',
+		'Aruba_Wireless_Controller.xml.gz',
+		'BayTech_PDU.xml.gz',
 		'Cacti_Stats.xml.gz',
 		'Cisco_Router.xml.gz',
 		'Citrix_NetScaler_VPX.xml.gz',
+		'ESXi_Device.xml.gz',
+		'Fortigate.xml.gz',
 		'Generic_SNMP_Device.xml.gz',
+		'HPE_iLO.xml.gz',
 		'Local_Linux_Machine.xml.gz',
+		'MikroTik_Device.xml.gz',
+		'MikroTik_Switch_SWOS.xml.gz',
+		'Motorola_SB6141.xml.gz',
 		'NetSNMP_Device.xml.gz',
 		'PING_Advanced_Ping.xml.gz',
+		'SNMP_Printer.xml.gz',
+		'SNMP_UPS.xml.gz',
 		'Synology_NAS.xml.gz',
 		'Windows_Device.xml.gz'
 	);
@@ -1108,7 +1127,7 @@ function install_full_sync() {
 					db_execute_prepared('UPDATE poller
 						SET last_sync = NOW()
 						WHERE id = ?',
-						array($id));
+						array($poller['id']));
 				} else {
 					log_install_debug('sync', 'Failed replication to Poller ' . $poller['id']);
 					$failed[] = $poller['id'];
