@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -142,7 +142,7 @@ if (!$forcerun) {
 switch ($type) {
 	case 'pmaster':
 		if (read_config_option('dsstats_enable') == 'on' || $forcerun) {
-			dsstats_master_handler($forcerun);
+			dsstats_master_handler($type, $forcerun);
 		}
 
 		break;
@@ -162,7 +162,7 @@ switch ($type) {
 	case 'bchild': // Launched by the boost process
 		$child_start = microtime(true);
 
-		dsstats_get_and_store_ds_avgpeak_values('daily', $thread_id);
+		dsstats_get_and_store_ds_avgpeak_values('daily', $type, $thread_id);
 
 		$total_time = microtime(true) - $child_start;
 
@@ -173,9 +173,9 @@ switch ($type) {
 		$child_start = microtime(true);
 
 		dsstats_debug(sprintf('Daily Stats Master Child %s Executing', $thread_id));
-		dsstats_get_and_store_ds_avgpeak_values('weekly', $thread_id);
-		dsstats_get_and_store_ds_avgpeak_values('monthly', $thread_id);
-		dsstats_get_and_store_ds_avgpeak_values('yearly', $thread_id);
+		dsstats_get_and_store_ds_avgpeak_values('weekly', $type, $thread_id);
+		dsstats_get_and_store_ds_avgpeak_values('monthly', $type, $thread_id);
+		dsstats_get_and_store_ds_avgpeak_values('yearly', $type, $thread_id);
 
 		$total_time = microtime(true) - $child_start;
 
@@ -212,9 +212,7 @@ function dsstats_insert_hourly_data_into_cache() {
 		ON DUPLICATE KEY UPDATE average=VALUES(average), peak=VALUES(peak)");
 }
 
-function dsstats_master_handler($forcerun) {
-	global $type;
-
+function dsstats_master_handler($type, $forcerun) {
 	/* read some important settings relative to timing from the database */
 	$major_time     = date('H:i:s', strtotime(read_config_option('dsstats_major_update_time')));
 	$daily_interval = read_config_option('dsstats_daily_interval');
@@ -229,7 +227,7 @@ function dsstats_master_handler($forcerun) {
 	// Insert new rows into cache
 	dsstats_insert_hourly_data_into_cache();
 
-	dsstats_log_statistics('HOURLY');
+	dsstats_log_statistics('HOURLY', $type);
 
 	/* see if boost is active or not */
 	$boost_active = read_config_option('boost_rrd_update_enable');
@@ -242,8 +240,10 @@ function dsstats_master_handler($forcerun) {
 		dsstats_debug('Skipping Periodic Rollup - Boost will handle the Periodic Roll-up Cycle');
 	} else {
 		if ($daily_interval == 'boost') {
-			cacti_log("WARNING: Daily update interval set to 'boost' and Boost Plugin Not Enabled, reseting to default of 1 hour", false, 'DSSTATS');
+			cacti_log("WARNING: Daily update interval set to 'boost' and boost not enabled, resetting to default of 1 hour", false, 'DSSTATS');
+
 			set_config_option('dsstats_daily_interval', 60);
+
 			$daily_interval = 60;
 		}
 
@@ -266,7 +266,7 @@ function dsstats_master_handler($forcerun) {
 				sleep(2);
 			}
 
-			dsstats_log_statistics('DAILY');
+			dsstats_log_statistics('DAILY', $type);
 		}
 	}
 
@@ -302,7 +302,7 @@ function dsstats_master_handler($forcerun) {
  */
 function display_version() {
 	$version = get_cacti_version();
-	print "Cacti Data Source Staitistcs Poller, Version $version " . COPYRIGHT_YEARS . PHP_EOL;
+	print "Cacti Data Source Statistics Poller, Version $version " . COPYRIGHT_YEARS . PHP_EOL;
 }
 
 /**
