@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -51,7 +51,7 @@ if ($config['poller_id'] > 1 && $config['connection'] == 'online') {
 }
 
 if (!remote_client_authorized()) {
-	print 'FATAL: You are not authorized to use this service';
+	print 'FATAL: Client authorization failed.  You are not authorized to use this service';
 	exit;
 }
 
@@ -130,7 +130,7 @@ function remote_agent_strip_domain($host) {
 }
 
 function remote_client_authorized() {
-	global $poller_db_cnn_id;
+	global $config, $poller_db_cnn_id;
 
 	/* don't allow to run from the command line */
 	$client_addr = get_client_addr();
@@ -151,9 +151,9 @@ function remote_client_authorized() {
 		$client_name = remote_agent_strip_domain($client_name);
 	}
 
-	$pollers = db_fetch_assoc('SELECT * FROM poller', true, $poller_db_cnn_id);
+	$pollers = db_fetch_assoc('SELECT * FROM poller WHERE disabled = ""', true, $poller_db_cnn_id);
 
-	if (cacti_sizeof($pollers)) {
+	if (cacti_sizeof($pollers) > 1) {
 		foreach($pollers as $poller) {
 			if (remote_agent_strip_domain($poller['hostname']) == $client_name) {
 				return true;
@@ -301,6 +301,11 @@ function poll_for_data() {
 	$poller_id      = get_nfilter_request_var('poller_id');
 	$return         = array();
 
+	/* ensure we have a valid poller_id */
+	if (!preg_match('/^[a-z0-9]+$/i', $poller_id)) {
+		return array();
+	}
+
 	$i = 0;
 
 	if (cacti_sizeof($local_data_ids)) {
@@ -382,7 +387,7 @@ function poll_for_data() {
 						);
 
 						if (function_exists('proc_open')) {
-							$cactiphp = proc_open(read_config_option('path_php_binary') . ' -q ' . $config['base_path'] . '/script_server.php realtime ' . $poller_id, $cactides, $pipes);
+							$cactiphp = proc_open(read_config_option('path_php_binary') . ' -q ' . $config['base_path'] . '/script_server.php realtime ' . cacti_escapeshellarg($poller_id), $cactides, $pipes);
 							$output = fgets($pipes[1], 1024);
 							$using_proc_function = true;
 						} else {
@@ -443,8 +448,8 @@ function run_remote_data_query() {
 function run_remote_discovery() {
 	global $config;
 
-	$poller_id = $config['poller_id'];
-	$network   = get_filter_request_var('network');
+	$poller_id = cacti_escapeshellarg($config['poller_id']);
+	$network   = cacti_escapeshellarg(get_filter_request_var('network'));
 	$php       = cacti_escapeshellcmd(read_config_option('path_php_binary'));
 	$path      = cacti_escapeshellarg(read_config_option('path_webroot') . '/poller_automation.php');
 
